package transferpipes;

import net.minecraft.server.MinecraftServer;
import net.minecraft.src.*;
import net.minecraft.entity.player.*;
import net.minecraft.item.*;
import net.minecraft.tileentity.*;
import net.minecraft.world.*;
import net.minecraft.nbt.*;
import net.minecraftforge.common.DimensionManager;
import buildcraft.api.tools.IToolWrench;
import buildcraft.core.ItemBuildCraft;
import buildcraft.transport.TileGenericPipe;

public class ItemRemoteConnector extends ItemBuildCraft implements IToolWrench {
	private int posx,posy,posz,dim;
	public int currentDim;
	private boolean flag;
	public ItemRemoteConnector(int itemid)
	{
		super(itemid);
		this.maxStackSize = 1;
	}

	public String getTextureFile() {
		return "/sprite/telepipes.png";
	}
	
	//ゲートGUI回避のためIToolWrenchをインプリメント
	@Override
	public boolean canWrench(EntityPlayer entityplayer, int x, int y, int z){
		return false;
	}
	
	@Override
	public void wrenchUsed(EntityPlayer entityplayer, int x, int y, int z){}

	@Override
    public boolean onItemUse(ItemStack itemstack, EntityPlayer entityplayer, World world, int x, int y, int z, int s, float i, float j, float k)
    {
		currentDim = entityplayer.dimension;
		if(!world.isRemote){
			readNBT(itemstack);
			PipeTransfer pipe;
			TileGenericPipe tile = getTile(world,x,y,z);
			if(tile != null){
				pipe = getPipe(tile);
				if(pipe != null){
					if(pipe.remoteMode){
						entityplayer.addChatMessage("Pipe Unconnecting");
						disconnectPipe(world, x, y, z);
						return true;
					}
					if(flag && !(x == posx && y == posy && z == posz)){
						if(connectPipe(world, x, y, z, dim)){
							entityplayer.addChatMessage("Connection Successfully");
							System.out.println("Connection Successfully");
							flag=false;
							writeNBT(itemstack);
							return true;
						}
						entityplayer.addChatMessage("Connection Refused");
						System.out.println("connection Refused");
						flag=false;
						writeNBT(itemstack);
						return true;
					}
					else{
						entityplayer.addChatMessage("Position Saved");
						System.out.println("position saved");
						posx=x;posy=y;posz=z;
						flag=true;
						dim=currentDim;
						writeNBT(itemstack);
					}
				}
			}
		}
        return true;
    }
	public TileGenericPipe getTile(World world, int x, int y, int z)
	{
		TileEntity tile = world.getBlockTileEntity(x, y, z);
		if(tile instanceof TileGenericPipe)
			return (TileGenericPipe)tile;
		return null;
	}
	
	public PipeTransfer getPipe(TileGenericPipe tile){
		if(tile.pipe instanceof PipeTransfer)
			return (PipeTransfer)tile.pipe;
		return null;
	}
	
	private boolean connectPipe(World world, int x, int y, int z, int dim)
	{
		TileGenericPipe tile1,tile2;
		tile1 = getTile(world,x,y,z);
		World toWorld = getWorldForDimension(dim);
		tile2 = getTile(toWorld,posx,posy,posz);
		boolean a = ((PipeTransfer)tile1.pipe).dimpipe;
		if((currentDim != dim) && !(((PipeTransfer)tile1.pipe).dimpipe && ((PipeTransfer)tile2.pipe).dimpipe))
			return false;
		if(tile1 != null && tile2 != null){
			if(tile1.pipe instanceof PipeItemsTransfer && tile2.pipe instanceof PipeItemsTransfer){
				setPipeRemote((PipeTransfer)tile1.pipe,posx,posy,posz,dim);
				tile1.scheduleRenderUpdate();
				tile1.scheduleNeighborChange();
				setPipeRemote((PipeTransfer)tile2.pipe,x,y,z,currentDim);
				tile2.scheduleRenderUpdate();
				tile2.scheduleNeighborChange();
				return true;
			}
			if(tile1.pipe instanceof PipeLiquidsTransfer && tile2.pipe instanceof PipeLiquidsTransfer){
				setPipeRemote((PipeTransfer)tile1.pipe,posx,posy,posz,dim);
				tile1.scheduleRenderUpdate();
				tile1.scheduleNeighborChange();
				((PipeLiquidsTransfer)tile1.pipe).ticker = 0;
				((PipeLiquidsTransfer)tile1.pipe).preceive = false;
				((PipeLiquidsTransfer)tile1.pipe).receive = false;
				setPipeRemote((PipeTransfer)tile2.pipe,x,y,z,currentDim);
				((PipeLiquidsTransfer)tile2.pipe).ticker = 0;
				((PipeLiquidsTransfer)tile2.pipe).preceive = true;
				((PipeLiquidsTransfer)tile2.pipe).receive = true;
				tile2.scheduleRenderUpdate();
				tile2.scheduleNeighborChange();
				return true;
			}
			if(tile1.pipe instanceof PipePowerTransfer && tile2.pipe instanceof PipePowerTransfer){
				setPipeRemote((PipeTransfer)tile1.pipe,posx,posy,posz,dim);
				tile1.scheduleRenderUpdate();
				tile1.scheduleNeighborChange();
				setPipeRemote((PipeTransfer)tile2.pipe,x,y,z,currentDim);
				tile2.scheduleRenderUpdate();
				tile2.scheduleNeighborChange();
				return true;
			}
		}
		return false;
	}
	private void disconnectPipe(World world, int x, int y, int z)
	{
		TileGenericPipe tile1,tile2;
		PipeTransfer pipe1 = null,pipe2 = null;
		tile1 = getTile(world,x,y,z);
		if(tile1 != null)
			pipe1 = getPipe(tile1);
		if(pipe1 != null){
			pipe1.remoteMode = false;
			pipe1.broadcastSignal = new boolean[]{false,false,false,false};
			tile1.scheduleRenderUpdate();
			tile1.scheduleNeighborChange();
			World toWorld = getWorldForDimension(pipe1.toDim);
			tile2 = getTile(toWorld, pipe1.toPosX, pipe1.toPosY, pipe1.toPosZ);
			if(tile2 != null)
				pipe2 = getPipe(tile2);
				if(pipe2 != null){
					if((x == pipe2.toPosX) && (y == pipe2.toPosY) && (z == pipe2.toPosZ)){
						pipe2.broadcastSignal = new boolean[]{false,false,false,false};
						pipe2.remoteMode = false;
						tile2.scheduleRenderUpdate();
						tile2.scheduleNeighborChange();
					}
				}
		}
	}
	private void setPipeRemote(PipeTransfer pipe, int x, int y, int z, int dim)
	{
		pipe.remoteMode = true;
		pipe.toPosX = x;
		pipe.toPosY = y;
		pipe.toPosZ = z;
		pipe.broadcastSignal = new boolean[]{false,false,false,false};
		pipe.toDim = dim;
	}
	
	private void readNBT(ItemStack itemstack){
		NBTTagCompound nbt = itemstack.getTagCompound();
		if(nbt != null){
			flag = nbt.getBoolean("flag");
			posx = nbt.getInteger("posx");
			posy = nbt.getInteger("posy");
			posz = nbt.getInteger("posz");
			dim = nbt.getInteger("dim");
			return;
		}
		flag = false;
	}
	
	private void writeNBT(ItemStack itemstack){
		NBTTagCompound nbt = new NBTTagCompound();
		nbt.setBoolean("flag", flag);
		nbt.setInteger("posx", posx);
		nbt.setInteger("posy", posy);
		nbt.setInteger("posz", posz);
		nbt.setInteger("dim", dim);
		itemstack.setTagCompound(nbt);
	}
	
	public World getWorldForDimension(int dim)
	{
		MinecraftServer server = MinecraftServer.getServer();
		return server.worldServerForDimension(dim);
	}
}
