/**
 * BuildCraft is open-source. It is distributed under the terms of the
 * BuildCraft Open Source License. It grants rights to read, modify, compile
 * or run the code. It does *NOT* grant the right to redistribute this software
 * or its modifications in any form, binary or source, except if expressively
 * granted by the copyright holder.
 */

package transferpipes;

import java.lang.reflect.Field;
import java.util.LinkedList;
import java.util.List;




import net.minecraftforge.common.ForgeDirection;

import buildcraft.BuildCraftTransport;
import buildcraft.api.core.Position;
import buildcraft.api.inventory.ISpecialInventory;
import buildcraft.api.power.IPowerProvider;
import buildcraft.api.power.IPowerReceptor;
import buildcraft.api.power.PowerFramework;
import buildcraft.api.tools.IToolWrench;
import buildcraft.api.transport.IPipe;
import buildcraft.api.transport.IPipedItem;
import buildcraft.api.transport.PipeManager;
import buildcraft.core.DefaultProps;
import buildcraft.core.utils.Utils;
import buildcraft.energy.TileEngine;
import buildcraft.transport.EntityData;
import buildcraft.transport.IItemTravelingHook;
import buildcraft.transport.IPipeTransportItemsHook;
import buildcraft.transport.Pipe;
import buildcraft.transport.PipeTransport;
import buildcraft.transport.PipeTransportItems;
import buildcraft.transport.TileGenericPipe;
import buildcraft.transport.pipes.PipeLogic;
import buildcraft.core.EntityPassiveItem;

import net.minecraft.inventory.*;
import net.minecraft.nbt.*;
import net.minecraft.world.*;
import net.minecraft.tileentity.*;
import net.minecraft.item.*;
import net.minecraftforge.client.MinecraftForgeClient;
import net.minecraftforge.common.ISidedInventory;

public class PipeItemsTransfer extends PipeTransfer implements IPipeTransportItemsHook,IPowerReceptor,IItemTravelingHook {

	private IPowerProvider powerProvider;
	public boolean set = false;
	public PipeItemsTransfer(int itemID) {
		super(new PipeTransportItems(), new PipeLogic(), itemID);
		powerProvider = PowerFramework.currentFramework.createPowerProvider();
		powerProvider.configure(50, 1, 64, 1, 64);
		powerProvider.configurePowerPerdition(64, 1);
		((PipeTransportItems) transport).travelHook = this;
	}
	@Override
	public String getTextureFile() {
		MinecraftForgeClient.preloadTexture("/sprite/telepipes.png");
		return "/sprite/telepipes.png";
	}
	

	
	@Override
	public int getTextureIndex(ForgeDirection direction) {
		if(remoteMode)
			return 1;
		else
			return 0;
	}
	
	@Override
	public void updateEntity(){

		super.updateEntity();
		if(remoteMode){
			teleportSignal();
		}

	}
	


	
	@Override
	public void setPowerProvider(IPowerProvider provider) {
		powerProvider = provider;
	}

	@Override
	public IPowerProvider getPowerProvider() {
		return powerProvider;
	}


	@Override
	public LinkedList<ForgeDirection> filterPossibleMovements(LinkedList<ForgeDirection> possibleOrientations, Position pos,
			IPipedItem item) {
		
		return possibleOrientations;
	}
	//木パイプよりほぼコピペ
	@Override
	public void doWork() {
		if (powerProvider.getEnergyStored() <= 0)
			return;

		World w = worldObj;

		int orient = 0;
		
		//吸い出し対象判定
		for(orient = 0;orient<=5;orient++){
			if(canExtractOrientations(w, orient))
				break;
			if(orient == 5)
				return;
		}
		System.out.println(orient);
		//追加ここまで
		Position pos = new Position(xCoord, yCoord, zCoord, ForgeDirection.values()[orient]);
		pos.moveForwards(1);
		TileEntity tile = w.getBlockTileEntity((int) pos.x, (int) pos.y, (int) pos.z);

		if (tile instanceof IInventory) {
         if (!PipeManager.canExtractItems(this, w, (int) pos.x, (int) pos.y, (int) pos.z))
            return;

         IInventory inventory = (IInventory) tile;

         ItemStack[] extracted = checkExtract(inventory, true, pos.orientation.getOpposite());
         if (extracted == null)
            return;

         for(ItemStack stack : extracted) {
            if (stack == null || stack.stackSize == 0) {
               powerProvider.useEnergy(1, 1, false);
                  continue;
            }

            Position entityPos = new Position(pos.x + 0.5, pos.y + Utils.getPipeFloorOf(stack), pos.z + 0.5,
                  pos.orientation.getOpposite());

            entityPos.moveForwards(0.5);

            IPipedItem entity = new EntityPassiveItem(w, entityPos.x, entityPos.y, entityPos.z, stack);

            ((PipeTransportItems) transport).entityEntering(entity, entityPos.orientation);
         }
      }
	}
	
	public ItemStack[] checkExtract(IInventory inventory, boolean doRemove, ForgeDirection from) {

		/// ISPECIALINVENTORY
		if (inventory instanceof ISpecialInventory) {
			ItemStack[] stacks = ((ISpecialInventory) inventory).extractItem(doRemove, from, (int)powerProvider.getEnergyStored());
			if (stacks != null && doRemove) {
				for (ItemStack stack : stacks) {
					if (stack != null) {
						powerProvider.useEnergy(stack.stackSize, stack.stackSize, true);
					}
				}
			}
			return stacks;
		}

		if (inventory instanceof ISidedInventory) {
			ISidedInventory sidedInv = (ISidedInventory) inventory;

			int first = sidedInv.getStartInventorySide(from);
			int last = first + sidedInv.getSizeInventorySide(from) - 1;

			IInventory inv = Utils.getInventory(inventory);

			ItemStack result = checkExtractGeneric(inv, doRemove, from, first, last);

			if (result != null)
				return new ItemStack[] { result };
		} else if (inventory.getSizeInventory() == 2) {
			// This is an input-output inventory

			int slotIndex = 0;

			if (from == ForgeDirection.DOWN || from == ForgeDirection.UP)
				slotIndex = 0;
			else
				slotIndex = 1;

			ItemStack slot = inventory.getStackInSlot(slotIndex);

			if (slot != null && slot.stackSize > 0)
				if (doRemove)
					return new ItemStack[] { inventory.decrStackSize(slotIndex, (int) powerProvider.useEnergy(1, slot.stackSize, true)) };
				else
					return new ItemStack[] { slot };
		} else if (inventory.getSizeInventory() == 3) {
			// This is a furnace-like inventory

			int slotIndex = 0;

			if (from == ForgeDirection.UP)
				slotIndex = 0;
			else if (from == ForgeDirection.DOWN)
				slotIndex = 1;
			else
				slotIndex = 2;

			ItemStack slot = inventory.getStackInSlot(slotIndex);

			if (slot != null && slot.stackSize > 0)
				if (doRemove)
					return new ItemStack[] { inventory.decrStackSize(slotIndex, (int) powerProvider.useEnergy(1, slot.stackSize, true)) };
				else
					return new ItemStack[] { slot };
		} else {
			// This is a generic inventory
			IInventory inv = Utils.getInventory(inventory);

			ItemStack result = checkExtractGeneric(inv, doRemove, from, 0, inv.getSizeInventory() - 1);

			if (result != null)
				return new ItemStack[] { result };
		}

		return null;
	}

	public ItemStack checkExtractGeneric(IInventory inventory, boolean doRemove, ForgeDirection from, int start, int stop) {
		for (int k = start; k <= stop; ++k)
			if (inventory.getStackInSlot(k) != null && inventory.getStackInSlot(k).stackSize > 0) {

				ItemStack slot = inventory.getStackInSlot(k);

				if (slot != null && slot.stackSize > 0)
					if (doRemove)
						return inventory.decrStackSize(k, (int) powerProvider.useEnergy(1, slot.stackSize, true));
					else
						return slot;
			}

		return null;
	}

	@Override
	public void entityEntered(IPipedItem item, ForgeDirection orientation) {
		item.setSpeed(Utils.pipeNormalSpeed * 30F);
		System.out.println("passive transport pipe");
	}

	@Override
	public void readjustSpeed(IPipedItem item) {
		((PipeTransportItems) transport).defaultReajustSpeed(item);
	}

	
	@Override
	public int powerRequest() {
		return getPowerProvider().getMaxEnergyReceived();
	}
	private boolean canExtractOrientations(World w, int orient)
	{
		Position pos = new Position(xCoord, yCoord, zCoord, ForgeDirection.values()[orient]);
		pos.moveForwards(1);
		TileEntity tile = w.getBlockTileEntity((int) pos.x, (int) pos.y, (int) pos.z);
		if (tile instanceof IInventory && !(tile instanceof TileEngine)) {
	         if (PipeManager.canExtractItems(this, w, (int) pos.x, (int) pos.y, (int) pos.z))
	            return true;
		}
		return false;
	}
	@Override
	public void drop(PipeTransportItems pipe, EntityData data) {
		IPipedItem item = data.item;
		//テレポート動作、entityEnteredやfilterPossibleMovementsに書くとループするので注意
		//entityEnteringが間接的に呼ばれてるため
		World toWorld = getWorldForDimension(toDim);
		TileGenericPipe tile = getTile(toWorld, toPosX, toPosY, toPosZ);
		if(tile != null){
			ForgeDirection orient = tile.pipe.getOpenOrientation().getOpposite();
			PipeTransfer pipeto = null;
			pipeto = getPipe(tile);
			
			if (orient != ForgeDirection.UNKNOWN && (pipeto != null)) {
				boolean illegal = (toDim != pipeto.toDim) && !(dimpipe && pipeto.dimpipe);
				if(remoteMode && pipeto.remoteMode && !illegal){
					item.setWorld(toWorld);
					item.setPosition(toPosX + 0.5, toPosY + Utils.getPipeFloorOf(null),toPosZ + 0.5);
					//IPipedItem passive = new EntityPassiveItem(worldObj, toPosX + 0.5, toPosY + Utils.getPipeFloorOf(stack),toPosZ + 0.5, stack);
					item.setSpeed(Utils.pipeNormalSpeed * 30F);
					tile.pipe.transport.entityEntering(item, orient);
					System.out.println("The item is teleported");
				}
			}
		}
	}
	@Override
	public void centerReached(PipeTransportItems pipe, EntityData data) {
	}
	@Override
	public void endReached(PipeTransportItems pipe, EntityData data,
			TileEntity tile) {
	}

}
