/**
 * BuildCraft is open-source. It is distributed under the terms of the
 * BuildCraft Open Source License. It grants rights to read, modify, compile
 * or run the code. It does *NOT* grant the right to redistribute this software
 * or its modifications in any form, binary or source, except if expressively
 * granted by the copyright holder.
 */

package transferpipes;

import net.minecraftforge.common.ForgeDirection;
import net.minecraftforge.liquids.ILiquidTank;
import net.minecraftforge.liquids.ITankContainer;
import net.minecraftforge.liquids.LiquidContainerRegistry;
import net.minecraftforge.liquids.LiquidStack;
import net.minecraftforge.liquids.LiquidTank;
import buildcraft.BuildCraftTransport;
import buildcraft.api.core.Position;
import buildcraft.api.power.IPowerProvider;
import buildcraft.api.power.IPowerReceptor;
import buildcraft.api.power.PowerFramework;
import buildcraft.api.transport.PipeManager;
import buildcraft.core.DefaultProps;
import buildcraft.core.network.TileNetworkData;
import buildcraft.core.utils.Utils;
import buildcraft.transport.PipeTransportLiquids.PipeSection;
import buildcraft.transport.TileGenericPipe;
import buildcraft.transport.IPipeTransportLiquidsHook;
import buildcraft.transport.PipeTransportLiquids.TransferState;
import buildcraft.transport.pipes.*;
import buildcraft.transport.Pipe;
import buildcraft.transport.PipeTransportLiquids;
import net.minecraft.inventory.*;
import net.minecraft.nbt.*;
import net.minecraft.tileentity.*;

public class PipeLiquidsTransfer extends PipeTransfer implements IPipeTransportLiquidsHook {

	public @TileNetworkData
	int liquidToExtract;

	protected boolean receive = false;
	protected boolean preceive = false;
	protected int ticker = 0;

	long lastMining = 0;
	boolean lastPower = false;

	public PipeLiquidsTransfer(int itemID) {
		super(new PipeTransportLiquids(), new PipeLogic(), itemID);
		((PipeTransportLiquids) transport).flowRate = 160;
		((PipeTransportLiquids) transport).travelDelay = 1;
	}

	@Override
	public void readFromNBT(NBTTagCompound nbt)
	{
		super.readFromNBT(nbt);
		preceive = nbt.getBoolean("receive");
		receive = this.preceive;
	}

	@Override
	public void writeToNBT(NBTTagCompound nbt)
	{
		super.writeToNBT(nbt);
		nbt.setBoolean("remoteMode", remoteMode);
		nbt.setBoolean("receive", preceive);
	}



	/**
	 * Extracts a random piece of item outside of a nearby chest.
	 */

	@Override
	public void updateEntity() {
		super.updateEntity();
		if(remoteMode)
			teleportSignal();
		//ループ対策
		if(this.receive)
			if(this.ticker <= 5)
				this.ticker++;
			else{
				TileGenericPipe tileto = getTile(getWorldForDimension(toDim), toPosX, toPosY, toPosZ);
				if(tileto !=null)
					if(tileto.pipe instanceof PipeLiquidsTransfer){
						PipeLiquidsTransfer pipe = (PipeLiquidsTransfer)tileto.pipe;
						this.receive = false;
						pipe.receive = true;
						pipe.ticker = 0;
				}
			}
		if (broadcastRedstone || worldObj.isBlockIndirectlyGettingPowered(xCoord, yCoord, zCoord) && !remoteMode)
			liquidToExtract = LiquidContainerRegistry.BUCKET_VOLUME;

		if (liquidToExtract > 0) {
			int orient;
			//吸い出し対象判定
			for(orient = 0;orient<=5;orient++){
				if(canExtractOrientations(orient))
					break;
				if(orient == 5){
					System.out.println(orient);
					return;
				}
			}
			Position pos = new Position(xCoord, yCoord, zCoord, ForgeDirection.values()[orient]);
			pos.moveForwards(1);

			TileEntity tile = worldObj.getBlockTileEntity((int) pos.x, (int) pos.y, (int) pos.z);

			if (tile instanceof ITankContainer) {
				ITankContainer container = (ITankContainer) tile;

				int flowRate = ((PipeTransportLiquids) transport).flowRate;

				LiquidStack extracted = container.drain(pos.orientation.getOpposite(), liquidToExtract > flowRate ? flowRate : liquidToExtract, false);

                int inserted = 0;
                if(extracted != null) {
                    inserted = ((PipeTransportLiquids) transport).fill(pos.orientation, extracted, true);

                    container.drain(pos.orientation.getOpposite(), inserted, true);
                }

				liquidToExtract -= inserted;
			}
		}
	}

	@Override
	public String getTextureFile() {
		return "/sprite/telepipes.png";
	}



	@Override
	public int getTextureIndex(ForgeDirection direction) {
		if(remoteMode)
			return 5;
		if (broadcastRedstone || worldObj != null && worldObj.isBlockIndirectlyGettingPowered(xCoord, yCoord, zCoord))
			return 6;
		else
			return 4;
	}

	private boolean canExtractOrientations(int orient)
	{
		Position pos = new Position(xCoord, yCoord, zCoord, ForgeDirection.values()[orient]);
		pos.moveForwards(1);
		TileEntity tile = worldObj.getBlockTileEntity((int) pos.x, (int) pos.y, (int) pos.z);
		if (tile instanceof ITankContainer && !(worldObj.getBlockId((int)pos.x, (int)pos.y, (int)pos.z) == DefaultProps.GENERIC_PIPE_ID)) {
	         if (PipeManager.canExtractLiquids(this, worldObj, (int) pos.x, (int) pos.y, (int) pos.z))
	            return true;
		}
		return false;
	}

	@Override
	public int fill(ForgeDirection from, LiquidStack resource, boolean doFill) {
		int filled;
		PipeLiquidsTransfer pipe = (PipeLiquidsTransfer)this.container.pipe;
		//転送処理
		if (pipe.remoteMode && !(this.receive)){
			TileGenericPipe tile = getTile(getWorldForDimension(toDim), pipe.toPosX, pipe.toPosY, pipe.toPosZ);
			if(tile != null){
				PipeTransfer pipeto = (PipeTransfer)tile.pipe;
				boolean illegal = (toDim != pipeto.toDim) && !(dimpipe && pipeto.dimpipe);
				if(pipeto.remoteMode && !illegal){
					filled = ((PipeTransportLiquids)tile.pipe.transport).fill(ForgeDirection.UNKNOWN, resource, doFill);
					return filled;
				}
			}
		}
		filled = ((PipeTransportLiquids)this.container.pipe.transport).getTanks(from)[from.ordinal()].fill(resource, doFill);


		/*if (filled > 0 && doFill && tankIndex != Orientations.Unknown.ordinal()){
			((PipeTransportLiquids)this.container.pipe.transport).transferState[tankIndex] = TransferState.Input;
			((PipeTransportLiquids)this.container.pipe.transport)inputTTL[tankIndex] = INPUT_TTL;
		}*/
		return filled;
	}
}
